% Generate some random RT data for the purposes of demonstrating method
RTdata = randn(225,1)*100+500; % mean RT of 500 ms with std of 100 ms
stim = repmat([1:9]',[25,1]); % in this example assume Fixed SART block
RTdata(3:18:225) = 0; % In our simulated data, let's say every second NoGo "3" trial was a commission error

% some obvious parameters:
fs = 1/1.439;  % sample rate of the RT timecourse - reciprocal of the inter trial-onset interval
totaltr = 225;  % total number of trials
segmentlen = 75;    % length of data segment for FFT, in trials
stepsize = 25;      % how much to step segment by - 25 gives an overlap of 50, so 7 segments 
FFTlen = 450;     % zero-pad to this length.

numsegs = floor((totaltr-segmentlen)/stepsize) +1;   % how many segments can we take from data, given segment length and step size?
minseg = 4;         % minimum acceptable number of segments with no big gaps
toomanygaps = 5;    % what's a big gap? i.e. how many bad trials (omission, impossibly fast or 3) in a row is unacceptable and should be rejected?

% additional criterion:
maxomissions = 34; % maximum number of omissions for subject to be included

F = [0:FFTlen/2]*fs/FFTlen; % frequency scale.
[blah,SARTpeak] = min(abs(F-fs/9)); % find closest freq point to the Principal SART Peak (PSP; fs/9)

% the range over which the area under curve is measured:
FFAUCrange = [SARTpeak+5 : length(F)]; % faster than the PSP, excluding 5 frequency bins just higher than the PSP itself
SFAUCrange = [2 : SARTpeak-5];          % Slower than the PSP, excluding 5 frequency bins just lower than the PSP itself; Also exclude 0 Hz component (by starting at 2)

    
%%%% Start the calculations

num_omissions = length(find(stim~=3 & RTdata==0));
num_commissions = length(find(stim==3 & RTdata~=0));
        
if num_omissions>maxomissions, error('This subject made too many omissions - exclude'); end
        
good = find(RTdata >= 100 & stim~=3);   % find 'good' trials
gaps = good(2:end)-good(1:end-1);    % the lengths of all gaps throughout 225 trials
clear gaps_seg
% for each segment n, note the size of the gaps
for n=1:numsegs
    good_seg = good(find(good>(n-1)*stepsize & good<=(n-1)*stepsize+segmentlen));
    gaps_seg{n} = good_seg(2:end)-good_seg(1:end-1) -1;    % the lengths of gaps in single segment (no gap is zero)
end
% Then linearly interpolate across all gaps (including omissions, <100ms trials and NoGo "3" trials):
for n=1:length(good)-1
    for m=1:gaps(n)-1
        RTdata(good(n)+m) = RTdata(good(n)) + m*(RTdata(good(n+1))-RTdata(good(n)))/gaps(n);
    end
end

% Now we're ready to compute the FFTs
seg_spec=zeros(length(F),numsegs);	% initialise the spactra to zero
keepseg = []; % keep track of which segments should be kept/rejected
for n=1:numsegs
    % If there are too many gaps in the segment that had to be interpolated, do not in clude that segment in the average
    % spectral estimate:
    if max(gaps_seg{n}) < toomanygaps, keepseg=[keepseg n]; end 
    % take out the nth 75-long segment and detrend first:
    segment = detrend(RTdata((n-1)*stepsize+[1:segmentlen]));
    % Hamming-window that segment, zero pad to length 450, and compute the squared magnitude (power) of the FFT:
    temp = (abs(fft([hamming(segmentlen).*segment ; zeros(FFTlen-segmentlen,1)]))/(segmentlen/2)).^2;
    seg_spec(:,n) = temp(1:length(F)); % store the result, up until the maximum frequency you want stored
end

if length(keepseg)<minseg, error('Not enough good segments - exclude subject'); end

% average over the FFTs of the 'good' segments (7 for an omission-less subject):
spec = mean(seg_spec(:,keepseg),2);
        
% 1st and 2nd half analysis:
spec1 = mean(seg_spec(:,keepseg(find(keepseg>=1 & keepseg<=3))),2);
spec2 = mean(seg_spec(:,keepseg(find(keepseg>=5 & keepseg<=7))),2);
        
PSP = spec(SARTpeak)./mean(spec(FFAUCrange)); % normalised principal SART peak amplitude
FFAUC = mean(spec(FFAUCrange));
SFAUC = mean(spec(SFAUCrange));
    
% By half:
PSP1 = spec1(SARTpeak)./mean(spec1(FFAUCrange));
PSP2 = spec2(SARTpeak)./mean(spec2(FFAUCrange));
FFAUC1 = mean(spec1(FFAUCrange));
FFAUC2 = mean(spec2(FFAUCrange));
SFAUC1 = mean(spec1(SFAUCrange));
SFAUC2 = mean(spec2(SFAUCrange));
    
% ExGaussian fit (Download files provided by Lacouture et al (2002))
RTdata4eg = RTdata(good);                            % ***** keep all non-zero and non-dodgy RTs for ExG fit.
R = simple_egfit(RTdata4eg);
mu = R(1); sig = R(2);  tau = R(3); 
% same thing for first and second halves:
RTdata4eg = RTdata(good(find(good<totaltr/2)));
R = simple_egfit(RTdata4eg);
mu1 = R(1); sig1 = R(2);  tau1 = R(3); 
RTdata4eg = RTdata(good(find(good>totaltr/2)));
R = simple_egfit(RTdata4eg);
mu2 = R(1); sig2 = R(2);  tau2 = R(3); 

